/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include "dps_demo.h"

#define TIME_INCREMENT 1.2


DPSshandle pset;
DPSshandle fset;
DPSshandle oset;
DPShandle  explosion_handle;

float sim_time = 0;


float simulation_time() {
  return sim_time;
}


unsigned int live_particles() {
  return dpsLiveParticles(pset);
}




void dps_init() {
  pset = dpsGenParticleSystem(1);
  fset = dpsGenForceSystem(1);
  oset = dpsGenObjectSystem(1);
  

  dpsNewForceSystem(fset);
    dpsNewKillOld(200.0f, DPS_FALSE);
    dpsNewGravity(0.0, -0.05, 0.0);
    dpsNewDamping(.99, .99, .99, 0.0, 50.0);
  dpsEndForceSystem();



  dpsColor3f(0.7f, 0.0f, 0.0f);
  dpsRGBADev(0.3f, 0.0f, 0.0f, 0.0f);


  dpsNewObjectSystem(oset);
    dpsNewCone(0.0, 18.0, 3.0, 0.0, 21.0, 3.0, 0.0, 0.3, DPS_SOURCE, 25.0, 1.1, 0.3);

    dpsNewPlane(0.0, 0.0, 0.0, 0.0, 1.0, 0.0, DPS_COLLIDE, 0.2, 0.2, 0.1);

    dpsNewSphere(0.0, 10.0, -5.0, 0.0, 8.0, DPS_COLLIDE,    0.1, 0.1, 0.1);
    dpsNewSphere(0.0, 0.0, 0.0, 0.0, 12.0, DPS_COLLIDE,    0.1, 0.1, 0.1);
    dpsNewSphere(4.0, 10.0, -1.0, 0.0, 6.0, DPS_COLLIDE,   0.3, 0.1, 0.1);
    dpsNewSphere(-4.0, 12.0, 0.0, 0.0, 6.0, DPS_COLLIDE,   0.1, 0.1, 0.1);
    dpsNewSphere(0.0, 0.0, 10.0, 0.0, 5.0, DPS_COLLIDE,     0.2, 0.1, 0.1);
  dpsEndObjectSystem();



  dpsTimeIncrementf(TIME_INCREMENT);
}








void dps_iterate() {
  dpsIterateParticles(pset, fset, oset);
  sim_time += TIME_INCREMENT;

  if(((int)(sim_time+1)) % 100 == 0) {
    if(explosion_handle) dpsRemoveForce(fset, explosion_handle);
    dpsNewForceSystem(fset);
      explosion_handle = dpsNewExplosion(0.0, 25, 3.0, 4, 8, 8, 1, 0.0);
    dpsEndForceSystem();
  }
}





void dps_display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  dpsDrawParticleSystem(DPS_LINES | DPS_COLOR, pset);
/*

  glColor3f(0.3, 0.2, 0.1);
  
  glPushMatrix();
  glTranslatef(0.0, 0.0, 0.0);
  glutSolidSphere(12, 15, 15);
  glPopMatrix();

  glPushMatrix();
  glTranslatef(4.0, 10.0, -1.0);
  glutSolidSphere(6, 15, 15);
  glPopMatrix();

  glPushMatrix();
  glTranslatef(-4.0, 12.0, 0.0);
  glutSolidSphere(6, 15, 15);
  glPopMatrix();
*/
}


