/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include "dps_demo.h"

#define TIME_INCREMENT 1.2
#define LIVE_PARTICLES 5000

DPSshandle pset;
DPSshandle fset;
DPSshandle oset;
float sim_time = 0;


float simulation_time() {
  return sim_time;
}


unsigned int live_particles() {
  return dpsLiveParticles(pset);
}



void collidefunc(DPShandle p, DPSshandle ps, DPSshandle fs, DPSshandle os) {
  DPSvector3f pos, vel;

  if (dpsLiveParticles(ps) < LIVE_PARTICLES) {
    dpsParticlePosition3fv(pos, p);
    dpsParticleVelocity3fv(vel, p);

    dpsNewParticleSystem(ps);
      dpsPosition3f(pos[0], pos[1],  pos[2]);
      dpsVelocity3f(vel[0] + get_rand(-1.0, 1.0),  -1.0*vel[1], vel[2] + get_rand(-1.0, 1.0));
      dpsNewParticle();
    dpsEndParticleSystem();
  }    
}




void dps_init() {
  pset = dpsGenParticleSystem(1);
  fset = dpsGenForceSystem(1);
  oset = dpsGenObjectSystem(1);
  

  dpsColor3f(0.8f, 0.7f, 0.2f);
  dpsRGBADev(0.2f, 0.3f, 0.2f, 0.0f);

  dpsCollideFunc(collidefunc);
  dpsPosition3f(0.0, 30.0, 20.0);
  dpsVelocity3f(get_rand(-0.2, 0.2), 0.0 , get_rand(-0.2, 0.2));


  dpsNewParticleSystem(pset);
    dpsNewParticle();
  dpsEndParticleSystem();



  dpsNewForceSystem(fset);
    dpsNewKillOld(120.0f, DPS_FALSE);
    dpsNewGravity(0.0, -0.07, 0.0);
  dpsEndForceSystem();





  dpsNewObjectSystem(oset);
    dpsNewPlane(0.0, 0.0, 0.0, 0.0, 1.0, 0.0, DPS_COLLIDE, 0.4, 0.8, 1.0);
  dpsEndObjectSystem();



  dpsTimeIncrementf(TIME_INCREMENT);
}








void dps_iterate() {
  dpsIterateParticles(pset, fset, oset);
  sim_time += TIME_INCREMENT;
}





void dps_display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  dpsDrawParticleSystem(DPS_LINES | DPS_COLOR, pset);
}


