/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include "utility.h"

/*
 * Definicao de algumas tabelas mapeadoras de funcoes.
 */
DPSboolean DPS_TABLES_LOADED  = DPS_FALSE;    // utilizado para q as tables
                                              // so sejam carregas uma vez
DPSfloat DPS_TABLE_ACOSINE[DPS_PRECISION_ACOSINE]; 
DPSfloat DPS_TABLE_COSINE[DPS_PRECISION_COSINE]; 
DPSfloat DPS_TABLE_SINE[DPS_PRECISION_SINE]; 







void dpsLoadTables() {
  unsigned long i;

  if(DPS_TABLES_LOADED) return;
  DPS_TABLES_LOADED = DPS_TRUE;

  for(i=0; i<DPS_PRECISION_ACOSINE; i++) { // argumento coseno
    DPS_TABLE_ACOSINE[i] =
      180.0f/M_PI * acos((DPSfloat)i*2.0f/DPS_PRECISION_ACOSINE - 1.0f);
  }

  for(i=0; i<DPS_PRECISION_COSINE; i++) {  //coseno
    DPS_TABLE_COSINE[i] = 
      cos((DPSfloat)i / DPS_PRECISION_COSINE * 360.0f * M_PI / 180.0f);
  }
  
  for(i=0; i<DPS_PRECISION_SINE; i++) {  //seno
    DPS_TABLE_SINE[i] =
      sin((DPSfloat)i / DPS_PRECISION_SINE * 360.0f * M_PI / 180.0f);
  }
}





inline DPSfloat dpsCos(DPSfloat a) {
  a = (a<0)?-1.0f*a:a;
  return 
    DPS_TABLE_COSINE[(unsigned long) ((a - ((unsigned long)(a / 360.0f)) * 360.0f) / 360.0f * DPS_PRECISION_COSINE)];
}



inline DPSfloat dpsSin(DPSfloat a) {
  if (a>=0) return  
    DPS_TABLE_SINE[(unsigned long) ((a - ((unsigned long)(a / 360.0f)) * 360.0f) / 360.0f * DPS_PRECISION_SINE)];

  a *= -1.0f;
  return -1.0f*DPS_TABLE_SINE[(unsigned long) ((a - ((unsigned long)(a / 360.0f)) * 360.0f) / 360.0f * DPS_PRECISION_SINE)];
}




inline DPSfloat dpsArgCos(DPSfloat a) {
  return DPS_TABLE_ACOSINE[
    (unsigned long) ((a + 1.0f)/2.0f * DPS_PRECISION_ACOSINE)];
}




inline DPSfloat dpsAngle2v(DPSvector3f v1, DPSvector3f v2) {
  return dpsArgCos((v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2]) /
                   (sqrt(v1[0] * v1[0] + v1[1] * v1[1] + v1[2] * v1[2]) *
                    sqrt(v2[0] * v2[0] + v2[1] * v2[1] + v2[2] * v2[2])));
}




inline float get_rand(float lower, float upper)
 {
        float rand_value;
        rand_value = rand();            //random integer
        rand_value /= RAND_MAX;         //float between 0 and 1
        rand_value *= upper - lower;    //pick number between upper & lower
        rand_value += lower;            //put number in the right place
        return rand_value;
}




inline DPSfloat* dpsSubVect(DPSvector3f result, DPSvector3f a, DPSvector3f b) {
  result[0] = a[0] - b[0];
  result[1] = a[1] - b[1];
  result[2] = a[2] - b[2];
  return result;
}


inline DPSfloat* dpsAddVect(DPSvector3f result, DPSvector3f a, DPSvector3f b) {
  result[0] = a[0] + b[0];
  result[1] = a[1] + b[1];
  result[2] = a[2] + b[2];
  return result;
}


inline DPSfloat dpsProdScalVect(DPSfloat *result, DPSvector3f a, DPSvector3f b) {
  return (*result = a[0]*b[0] + a[1]*b[1] + a[2]*b[2]);
}

 

inline DPSfloat* dpsProdVect(DPSvector3f result, DPSvector3f a, DPSvector3f b) {
  result[0] = a[1] * b[2] - b[1] * a[2];
  result[1] = a[2] * b[0] - a[0] * b[2];
  result[2] = a[0] * b[1] - a[1] * b[0];
  return result;
} 



inline DPSfloat* dpsMulVectConst(DPSvector3f result, DPSvector3f a, DPSfloat c){
  result[0] = a[0] * c;
  result[1] = a[1] * c; 
  result[2] = a[2] * c;
  return result;
}

inline DPSfloat* dpsDivVectConst(DPSvector3f result, DPSvector3f a, DPSfloat i){
  DPSfloat  c = 1/i;
  result[0] = a[0] * c;
  result[1] = a[1] * c; 
  result[2] = a[2] * c;
  return result;
}


inline DPSfloat dpsLengthVect(DPSfloat *result, DPSvector3f a){
  return *result = sqrtf(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
}

inline DPSfloat dpsSqrLengthVect(DPSfloat *result, DPSvector3f a){
  return (*result = a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
}


inline DPSfloat* dpsNormalizeVect(DPSvector3f result, DPSvector3f a){
  DPSfloat inv = 1.0f / (DPSfloat)sqrtf((DPSfloat)(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]));
 
  result[0] = a[0] * inv;
  result[1] = a[1] * inv; 
  result[2] = a[2] * inv;  
  return result;  
}


// funcoes utilizadas para gerar pontos aleatorios


inline DPSfloat dpsRandomNormalDist(DPSfloat avg, DPSfloat stdev) {
  if(stdev==0) return avg;
  return avg + get_rand(-1.0f*stdev, stdev);
}



inline DPSfloat dpsNormalRand(DPSfloat sigma) {
  DPSfloat y;
  if(sigma==0) return 0;
  return get_rand(-1.0f*sigma, sigma);
/*
  do{
    if (!(y = dpsRand48())) y += DPS_EPSILON;
    y = -1.0f * ((DPSfloat)log(y));
  } while(dpsRand48() > ((DPSfloat)exp(-1.0f*dpsSqr(y - 1.0f)) * 0.5f));
  
  if(rand() % 2 == 0)
    return y * sigma;// * DPS_ONE_OVER_SIGMA_EXP;
  else
    return -1.0f* y * sigma; //* DPS_ONE_OVER_SIGMA_EXP;  
*/    
}




inline DPSfloat* dpsRandVect(DPSvector3f randvect){
  randvect[0] = dpsRand48();
  randvect[1] = dpsRand48();
  randvect[2] = dpsRand48();
  return randvect;
}  

  
inline DPSfloat* dpsHalfVect(DPSvector3f halfvect){
  halfvect[0] = 0.5;
  halfvect[1] = 0.5;
  halfvect[2] = 0.5;
  return halfvect;
}

// acabam aqui

