/*
 * Dynamic Particle System Library (DPS)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _UTILITY_H_
#define _UTILITY_H_


#include <math.h>
#include <stdlib.h>
#include "types.h"




/*
 *  definicao da raiz quadrada
 */ 
#define   sqrtf(args)     ((DPSfloat)sqrt(args))

/* retorna o numero random [0,1]  */
#define   dpsRand48()     ((DPSfloat) drand48())

//retorna o quadrado de um numero
#define   dpsSqr(X)        ((DPSfloat)X * X)



// retorna um float com distribuicao normal de parametro Sigma
inline DPSfloat dpsNormalRand(DPSfloat sigma);
// retorna um vector com as suas componentes atribuidas aleatoriamenre entre [0,1]
inline DPSfloat* dpsRandVect();
//retorna um vector com as suas componentes iguais a 0.5
inline DPSfloat* dpsHalfVect();



inline DPSfloat dpsRandomNormalDist(DPSfloat avg, DPSfloat stdev);


/*
 *  Retorna um float entre lower e upper
 */ 
inline DPSfloat get_rand(DPSfloat lower, DPSfloat upper);



/*
 * Carrega as tabelas mapeadoras de funcoes.
 * Se isto nao for feito as novas funcoes matematicas retornam sempre 0.0f.
 */ 
void dpsLoadTables();


/*
 * Funcoes matematicas sobre vectores, e outras cenas...
 * 
 * Nota importante: Todas as funcoes sobre vectores exigem um argumento 
 * de retorno por referencia, mas esse mesmo resultado eh retornado
 * de forma a permitir composicao de funcoes.
 * 
 */ 



/*
 * Retorna o argumento do coseno em graus
 * e nao radianos. (a -> [-1, 1])
 *
 * Se se pretender o resultado em radianos
 * M_PI/180.0f * dpsArgCos(x);
 *
 * NOTA: nao testei isto do radianos... mas deve funcionar... 
 * experimente quem precisar.
 *
 *
 * As funcoes trigonometricas mapeadas em memoria,
 * sao 18% mais rapidas que as incluidas na math.h
 *
 */ 
inline DPSfloat dpsArgCos(DPSfloat a);

/*
 *  Retorna o coseno do angulo a em graus 
 */
inline DPSfloat dpsCos(DPSfloat a);


/*
 *  Retorna o seno do angulo a em graus 
 */
inline DPSfloat dpsSin(DPSfloat a);


/*
 *  Retorna o angulo em graus entre dois vectores
 */
inline DPSfloat dpsAngle2v(DPSvector3f v1, DPSvector3f v2);


/*
 *  result = a - b (diferenca de vectores)
 */ 
inline DPSfloat* dpsSubVect(DPSvector3f result, DPSvector3f a, DPSvector3f b);


/*
 *  result = a + b (soma de dois vectores)
 */ 
inline DPSfloat* dpsAddVect(DPSvector3f result, DPSvector3f a, DPSvector3f b);


/*
 *  result = a.b (Produto escalar de dois vectores)
 *  o resultado e'um escalar(float no caso)
 */ 
inline DPSfloat dpsProdScalVect(DPSfloat *result, DPSvector3f a, DPSvector3f b);


/*
 *  result = a x b (produto vectorial de dois vectores)
 *  identico ao calculo dum determinante de uma matriz 3x3
 *  utilizado para calcular um vector normal a outros dois
 */ 
inline DPSfloat* dpsProdVect(DPSvector3f result, DPSvector3f a, DPSvector3f b);

/* 
 * Multiplicacao dum vector por uma constante
 */
inline DPSfloat* dpsMulVectConst(DPSvector3f result, DPSvector3f a, DPSfloat c);

/* 
 * Divisao dum vector por uma constante
 */
inline DPSfloat* dpsDivVectConst(DPSvector3f result, DPSvector3f a, DPSfloat c);

/*
 * Norma de um vector
 */
inline DPSfloat dpsLengthVect(DPSfloat *result, DPSvector3f a);

/*
 * calculo da soma dos quadrados de cada componente do vector
 * ou seja o quadrado da norma
 */
inline DPSfloat dpsSqrLengthVect(DPSfloat *result, DPSvector3f a);

/*
 * Normalizacao de um vector (divide o vector pela sua norma
 * i.e, retorna um vector de norma
 */
inline DPSfloat* dpsNormalizeVect(DPSvector3f result, DPSvector3f a);


#endif

