/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "dps_demo.h"
#include <ctype.h>
/*
 * Setup
 */
 
#define WINDOW_WIDTH                    500
#define WINDOW_HEIGHT                   400
#define POINT_SIZE                      1
#define LINE_SIZE                       1
#define PERSPECTIVE_LENGHT              20000
#define MENU_WIDTH                      100
int LIGHTS        = 0;
int CAM_MOVE_SRC  = 1;
float CAM_DST_X   = 0.0f;    // camera stuff
float CAM_DST_Y   = 0.0f;
float CAM_DST_Z   = 0.0f;
float CAM_SRC_X   = 10.0f;
float CAM_SRC_Y   = 10.0f;
float CAM_SRC_Z   = 80.0f;
int AXIS = 0;

float fps;
int frames = 0, time, timebase;
 

/*
 *  Keyboard stuff
 */

void keyboardSpecial(int key, int x, int y) {
  if(CAM_MOVE_SRC)
    switch(key) {
      case GLUT_KEY_LEFT     : CAM_SRC_X--;   break;
      case GLUT_KEY_RIGHT    : CAM_SRC_X++;   break;
      case GLUT_KEY_UP       : CAM_SRC_Y++;   break;
      case GLUT_KEY_DOWN     : CAM_SRC_Y--;   break;
      case GLUT_KEY_PAGE_UP  : CAM_SRC_Z--;   break;
      case GLUT_KEY_PAGE_DOWN: CAM_SRC_Z++;   break;
  } else 
    switch(key) {
      case GLUT_KEY_LEFT     : CAM_DST_X--;   break;
      case GLUT_KEY_RIGHT    : CAM_DST_X++;   break;
      case GLUT_KEY_UP       : CAM_DST_Y++;   break;
      case GLUT_KEY_DOWN     : CAM_DST_Y--;   break;
      case GLUT_KEY_PAGE_UP  : CAM_DST_Z--;   break;
      case GLUT_KEY_PAGE_DOWN: CAM_DST_Z++;   break;
    }  
    
    
    
  glLoadIdentity();
  gluLookAt(CAM_SRC_X, CAM_SRC_Y, CAM_SRC_Z, 
            CAM_DST_X, CAM_DST_Y, CAM_DST_Z, 0, 1, 0); 
}




void keyboardNormal(unsigned char key, int x, int y) {
  key=toupper(key);
  switch(key) {
    case 'F': printf("Frames per Second: %.1f\n", fps);  break;
    case 'P': printf("Live Particles: %d\n", live_particles()); break;
    case 'T': printf("Simulation Time: %.1f\n", simulation_time()); break;
    case 'A': 
      AXIS = ! AXIS;
      printf("Axis: %s\n", AXIS?"on":"off"); 
      break;
    case 'L': 
      LIGHTS = ! LIGHTS;
      printf("Lights: %s\n", LIGHTS?"on":"off"); 
      if(LIGHTS) glEnable(GL_LIGHTING);
      else glDisable(GL_LIGHTING);
      
      break;
    case 'C': 
      CAM_MOVE_SRC = !CAM_MOVE_SRC;
      printf("Camera Move: %s\n", CAM_MOVE_SRC?"src":"dst"); 
      break;
  }
}



void calc_fps() {
  frames++;
  time = glutGet(GLUT_ELAPSED_TIME);
  if(time - timebase > 1000) {
    fps      = frames*1000.0/(time-timebase);
    timebase = time;
    frames   = 0;
  }
}




/*
 *  Display Inits
 */


void init() {
  GLfloat mat_specular[]   = {0.2, 0.2, 0.2, 1.0 };
  GLfloat mat_shininess[]  = {0.5};
  GLfloat light_position[] = {200.0, 200.0, 200.0, 0.0 };
  GLfloat white_light[]    = {0.2, 0.2, 0.7, 0.0 };


  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_SMOOTH);
  
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);

  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST);

  if(LIGHTS) glEnable(GL_LIGHTING);


  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_POINT_SMOOTH);

  glPointSize(POINT_SIZE);
  glLineWidth(LINE_SIZE);
}



void reshape(int w, int h){
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);

  glLoadIdentity();
  gluPerspective(60.0, (GLfloat)w / (GLfloat) h, 1.0, PERSPECTIVE_LENGHT);
  glMatrixMode(GL_MODELVIEW);

  glLoadIdentity();
  gluLookAt(CAM_SRC_X, CAM_SRC_Y, CAM_SRC_Z, 
            CAM_DST_X, CAM_DST_Y, CAM_DST_Z, 0, 1, 0); 
}





void display() {
  calc_fps();
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  dps_display();

  if(AXIS) {
    glBegin(GL_LINES); // desenhar eixos
      glColor3f(.5, .5, .5);
      glVertex3f(-30.0, 0.0, 0.0);
      glVertex3f(30.0, 0.0, 0.0);
      glVertex3f(0.0, -50.0, 0.0);
      glVertex3f(0.0, 50.0, 0.0);
      glVertex3f(0.0, 0.0, -30.0);
      glVertex3f(0.0, 0.0, 30.0);
    glEnd();
  }

  glutSwapBuffers();
}




void iterate() {
  dps_iterate();
  glutPostRedisplay();
}





/*
 *
 *
 *
 *
 */

int main(int argc, char **argv) {

  dps_init();

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA); 
  glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
  glutCreateWindow("DPS demonstration");
  glutDisplayFunc(display);
  glutIdleFunc(iterate);

  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboardNormal);
  glutSpecialUpFunc(keyboardSpecial);

  init();


  glutMainLoop();
  return 0;
}


